<html>
{# Should include some simple CSS head stuff here and like have a base template... what a mess #}
<body>

<p>Welcome to the {{settings.SITE.title}} site!</p>

<ul>
  # not yet sure how to formulate this
  {#
  {{obj|overview}}
  {{obj|permalink}}
  {{obj|editlink}}
  #}
  {% for di in rt.models.cal.Event.objects.all() %}
  {% for obj in rt.models.cal.Event.objects.all() %}
  <li>
    {{tostring(ar.obj2html(obj))}}</li>
{% endfor %}
</ul>

</body>
</html>
