{% extends "publisher/default.pub.html" %}
{%- block toolbar %}
    <p align="right">
        {{ tostring(ar.obj2html(obj, "Edit")) }}
        {# <a href="{{obj.publisher_url()}}">permalink</a> #}
        <a href="/login/">Sign in</a>
    </p>
{%- endblock %}

{%- block header %}
    <h1>{{ model._meta.verbose_name_plural }}</h1>
{%- endblock %}

{%- block main %}
    <p>The following {{ model._meta.verbose_name_plural }} are available:
        {% for obj in model.objects.all() %}
            {% include model.list_template %}
        {% endfor %}
    </p>
{%- endblock %}
